﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace ControlContainerEditor_CS
{
    public partial class NumberControl : UserControl
    {
        #region Private Members
        const int minValue = -100;
        const int maxValue = 100;
        readonly static Size preferredSize = new Size(86, 49);

        private int? value = 1;
        private bool buttonsVisible = true;

        #endregion //Private Members

        #region Events

        public event EventHandler ValueChanged;

        #endregion // Events

        #region Constructor
        public NumberControl()
        {
            InitializeComponent();
        }
        #endregion //Constructor

        #region Overrides

        #region OnResize
        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            int buttonLeft;
            if (this.ButtonsVisible)
            {
                int buttonWidth = this.btnUp.Width;
                int buttonHeight = this.Height / 2;
                buttonLeft = this.Width - buttonWidth;

                this.btnUp.SetBounds(buttonLeft, 0, buttonWidth, buttonHeight);
                this.btnDown.SetBounds(buttonLeft, this.btnUp.Bounds.Bottom + 1, buttonWidth, buttonHeight);
            }
            else
                buttonLeft = this.Bounds.Width;

            this.label1.SetBounds(0, 0, buttonLeft - 1, this.Height);
        }
        #endregion //OnResize

        #region GetPreferredSize
        public override Size GetPreferredSize(Size proposedSize)
        {
            return NumberControl.preferredSize;            
        }
        #endregion //GetPreferredSize

        #region Text
        public override string Text
        {
            get
            {
                return this.value.ToString();
            }
            set
            {
                int newValue = 0;
                bool success = int.TryParse(value, out newValue);
                if (success)
                    this.Value = newValue;
                else
                    this.Value = null;
            }
        }
        #endregion //Text

        #endregion //Overrides

        #region Public Properties

        #region ButtonsVisible
        public bool ButtonsVisible
        {
            get { return this.buttonsVisible; }
            set
            {
                this.buttonsVisible = value;

                this.btnUp.Visible = this.buttonsVisible;
                this.btnDown.Visible = this.buttonsVisible;
            }
        }
        #endregion //ButtonsVisible

        #region Value
        public int? Value
        {
            get { return this.value; }
            set 
            {
                if (value > maxValue || value < minValue)
                    return;

                this.value = value;

                this.RefreshLabel();

                if (this.ValueChanged != null)
                    this.ValueChanged(this, EventArgs.Empty);
            }
        }
        #endregion //Value

        #endregion //Public Properties

        #region Private Methods
        private void RefreshLabel()
        {
            this.label1.Text = this.value.ToString();
        }
        #endregion //Private Methods

        #region Event Handlers
        private void btn_Click(object sender, EventArgs e)
        {
            int increment = sender == this.btnUp
                ? 1
                : -1;

            int v = this.Value == null ? 0 : (int)this.Value;
            this.Value = v + increment;
        }
        #endregion //Event Handlers
    }
}
